/**
 * \file variable.h
 *
 * \brief Dclaration des fonctions et macros pour la gestion des variables
 * \author Denis MARTIN
 * \date 2017-2019
 */


/*

Copyright (C) 2017, 2019 Denis MARTIN

---------------------------------------

This file is part of MinD.

MinD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

MinD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MinD.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD.

MinD est un logiciel libre : il peut tre redistribu et/ou modifi
selon les termes de la Licence Publique Gnrale GNU telle que publie par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultrieure.

MinD est distribu dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Gnrale GNU pour plus de dtails.

Une copie de la Licence Publique Gnrale GNU devrait tre fournie
avec MinD. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#ifndef VARIABLE_INCLUDED
#define VARIABLE_INCLUDED

#include "bricks.h"


/**
 * \struct _MinDVar
 * \brief Structure de stockage d'une variable
 *
 * Cette structure permet de stocker les diffrents lments d'une variable,
 * comme son nom, son type, sa valeur et de chaner cette variable avec les autres
 * variables dclares.
 */
struct _MinDVar
{
  _TCHAR type;                  /**< type de variable : L (logique) / C (chane) / N (numrique) */
  _TCHAR *name;                 /**< nom de la variable */
  _TCHAR *value;                /**< valeur de la variable sous forme de chane,  convertir selon le type */
  int id;                     /**< identifiant  du contrle graphique ventuellement li  la variable */
  struct _MinDVar *previous;  /**< pointeur vers la variable prcdente */
  struct _MinDVar *next;      /**< pointeur vers la variable suivante */
};

typedef struct _MinDVar MinDVar;

MinDVar *mindVarGlobal;

MinDVar *MinDVariable_Fill(MinDVar **mindVarPtr, int type, _TCHAR *name, int id, _TCHAR *value);
MinDVar *MinDVariable_Retrieve(MinDVar **mindVarPtr, _TCHAR *name, int id);
int MinDVariable_FreeAll(MinDVar **mindVarPtr, FILE *fileOut);
int MinDVariable_Clean(MinDVar **mindVarPtr);
_TCHAR *MinDVariable_ReplaceVariableInString(MinDVar **mindVarPtr, _TCHAR *text);

MinDVar *MinDVariable_Declaration(MinDVar **mindVarPtr, _TCHAR *line);


#endif
